@@ -34,10 +34,14 @@ class Handler(WSGIRequestHandler):
 
     def __init__(self, socket, address, server, cb_request_received):
         self.cb_request_received = cb_request_received
+        self.env = None
         super().__init__(socket, address, server)
 
     def handle_one_request(self):
         try:
+            # Handler may be used for several requests, so we must reset
+            # environment overrides each time
+            self.env = None
             super().handle_one_request()
         finally:
             self.close_connection = True
@@ -50,10 +54,21 @@ class Handler(WSGIRequestHandler):
         self.server.sockets_waiting_for_request.remove(socket)
 
         if success and self.cb_request_received:
-            self.cb_request_received(self.get_environ())
+            self.cb_request_received(self._get_mutable_environ())
 
         return success
 
+    def _get_mutable_environ(self):
+        if self.env is None:
+            self.env = super().get_environ()
+        return self.env
+
+    def get_environ(self):
+        if self.env is None:
+            self.env = super().get_environ()
+        env_clone = dict(self.env)
+        return env_clone
+
     def log_message(self, format, *args):
         LOGGER.info(format, *args)
 
@@ -144,7 +159,9 @@ class Server:
             if not self.running:
                 break
 
+            starttime = time()
             event = Event()
+
             t = Thread(target=self._run_one_poll, args=(event, callbacks))
             t.start()
             event.wait()
